'use strict';
/*
 Copyright 2012-2015, Yahoo Inc.
 Copyrights licensed under the New BSD License. See the accompanying LICENSE file for terms.
 */
const LcovOnly = require('../lcovonly');

class TextLcov extends LcovOnly {
    constructor(opts) {
        super({
            ...opts,
            file: '-'
        });
    }
}

module.exports = TextLcov;
